@@ -28,13 +28,11 @@ KeyedHashVector = collections.namedtuple(
 
 
 def check_backend_support(backend, item):
-    supported = item.keywords.get("supported")
-    if supported:
-        for mark in supported:
-            if not mark.kwargs["only_if"](backend):
-                pytest.skip("{0} ({1})".format(
-                    mark.kwargs["skip_message"], backend
-                ))
+    for mark in item.node.iter_markers("supported"):
+        if not mark.kwargs["only_if"](backend):
+            pytest.skip("{0} ({1})".format(
+                mark.kwargs["skip_message"], backend
+            ))
 
 
 @contextmanager
@@ -136,7 +134,7 @@ def load_hash_vectors(vector_data):
             # string as hex 00, which is of course not actually an empty
             # string. So we parse the provided length and catch this edge case.
             msg = line.split(" = ")[1].encode("ascii") if length > 0 else b""
-        elif line.startswith("MD"):
+        elif line.startswith("MD") or line.startswith("Output"):
             md = line.split(" = ")[1]
             # after MD is found the Msg+MD (+ potential key) tuple is complete
             if key is not None:
@@ -437,7 +435,7 @@ def load_fips_dsa_sig_vectors(vector_data):
     return vectors
 
 
-# http://tools.ietf.org/html/rfc4492#appendix-A
+# https://tools.ietf.org/html/rfc4492#appendix-A
 _ECDSA_CURVE_NAMES = {
     "P-192": "secp192r1",
     "P-224": "secp224r1",
